@@ -14,7 +14,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+"""
+Reflection module.
+
+.. versionadded:: 1.1
+"""
+
 import inspect
+import logging
 import types
 
 import six
@@ -30,8 +37,26 @@ except AttributeError:
 _BUILTIN_MODULES = ('builtins', '__builtin__', '__builtins__', 'exceptions')
 
 
+LOG = logging.getLogger(__name__)
+
+
+if six.PY3:
+    Parameter = inspect.Parameter
+    Signature = inspect.Signature
+    get_signature = inspect.signature
+else:
+    # Provide an equivalent but use funcsigs instead...
+    import funcsigs
+    Parameter = funcsigs.Parameter
+    Signature = funcsigs.Signature
+    get_signature = funcsigs.signature
+
+
 def get_members(obj, exclude_hidden=True):
-    """Yields the members of an object, filtering by hidden/not hidden."""
+    """Yields the members of an object, filtering by hidden/not hidden.
+
+    .. versionadded:: 2.3
+    """
     for (name, value) in inspect.getmembers(obj):
         if name.startswith("_") and exclude_hidden:
             continue
@@ -44,39 +69,40 @@ def get_member_names(obj, exclude_hidden=True):
             get_members(obj, exclude_hidden=exclude_hidden)]
 
 
-def get_class_name(obj, fully_qualified=True):
+def get_class_name(obj, fully_qualified=True, truncate_builtins=True):
     """Get class name for object.
 
-    If object is a type, fully qualified name of the type is returned.
-    Else, fully qualified name of the type of the object is returned.
-    For builtin types, just name is returned.
+    If object is a type, returns name of the type. If object is a bound
+    method or a class method, returns its ``self`` object's class name.
+    If object is an instance of class, returns instance's class name.
+    Else, name of the type of the object is returned. If fully_qualified
+    is True, returns fully qualified name of the type. For builtin types,
+    just name is returned. TypeError is raised if can't get class name from
+    object.
     """
+    if inspect.isfunction(obj):
+        raise TypeError("Can't get class name.")
+
+    if inspect.ismethod(obj):
+        obj = get_method_self(obj)
     if not isinstance(obj, six.class_types):
         obj = type(obj)
-    try:
-        built_in = obj.__module__ in _BUILTIN_MODULES
-    except AttributeError:
-        pass
-    else:
-        if built_in:
-            try:
-                return obj.__qualname__
-            except AttributeError:
-                return obj.__name__
-    pieces = []
-    try:
-        pieces.append(obj.__qualname__)
-    except AttributeError:
-        pieces.append(obj.__name__)
-    if fully_qualified:
+    if truncate_builtins:
         try:
-            pieces.insert(0, obj.__module__)
-        except AttributeError:
+            built_in = obj.__module__ in _BUILTIN_MODULES
+        except AttributeError:  # nosec
             pass
-    return '.'.join(pieces)
+        else:
+            if built_in:
+                return obj.__name__
+    if fully_qualified and hasattr(obj, '__module__'):
+        return '%s.%s' % (obj.__module__, obj.__name__)
+    else:
+        return obj.__name__
 
 
-def get_all_class_names(obj, up_to=object):
+def get_all_class_names(obj, up_to=object,
+                        fully_qualified=True, truncate_builtins=True):
     """Get class names of object parent classes.
 
     Iterate over all class names object is instance or subclass of,
@@ -87,7 +113,9 @@ def get_all_class_names(obj, up_to=object):
         obj = type(obj)
     for cls in obj.mro():
         if issubclass(cls, up_to):
-            yield get_class_name(cls)
+            yield get_class_name(cls,
+                                 fully_qualified=fully_qualified,
+                                 truncate_builtins=truncate_builtins)
 
 
 def get_callable_name(function):
@@ -141,31 +169,51 @@ def get_method_self(method):
 
 
 def is_same_callback(callback1, callback2, strict=True):
-    """Returns if the two callbacks are the same."""
+    """Returns if the two callbacks are the same.
+
+    'strict' arg has no meaning for python 3.8 onwards and will
+    always return the equality of both callback based on 'self'
+    comparison only.
+    """
     if callback1 is callback2:
         # This happens when plain methods are given (or static/non-bound
         # methods).
         return True
     if callback1 == callback2:
+        # NOTE(gmann): python3.8 onward, comparison of bound methods is
+        # changed. It no longer decide the bound method's equality based
+        # on their bounded objects equality instead it checks the identity
+        # of their '__self__'. So python3.8 onward, two different bound
+        # methods are no longer equal even __eq__ method return True.
+        # Or in other term, 'strict' arg has no meaning from python 3.8
+        # onwards above if condition never satisfy if both callback are
+        # bounded to two different objects.
+        # For backward compatibility for python <3.8, we can keep the 'strict'
+        # arg and the below code of comparing 'self' and once minimum
+        # supported python version is 3.8 we can remove both because python
+        # 3.8 onward == operator itself checks identity of 'self'.
+        # Ref bug: https://bugs.launchpad.net/oslo.utils/+bug/1841072
         if not strict:
+            LOG.warning('"strict" arg is deprecated because it no '
+                        'longer work for python 3.8 onwards')
             return True
-        # Two bound methods are equal if functions themselves are equal and
-        # objects they are applied to are equal. This means that a bound
-        # method could be the same bound method on another object if the
-        # objects have __eq__ methods that return true (when in fact it is a
-        # different bound method). Python u so crazy!
+        # Until python 3.7, two bound methods are equal if functions
+        # themselves are equal and objects they are applied to are equal.
+        # This means that a bound method could be the same bound method on
+        # another object if the objects have __eq__ methods that return true
+        # (when in fact it is a different bound method). Python u so crazy!
         try:
             self1 = six.get_method_self(callback1)
             self2 = six.get_method_self(callback2)
             return self1 is self2
-        except AttributeError:
+        except AttributeError:  # nosec
             pass
     return False
 
 
 def is_bound_method(method):
     """Returns if the given method is bound to an object."""
-    return bool(get_method_self(method))
+    return get_method_self(method) is not None
 
 
 def is_subclass(obj, cls):
@@ -173,19 +221,6 @@ def is_subclass(obj, cls):
     return inspect.isclass(obj) and issubclass(obj, cls)
 
 
-def _get_arg_spec(function):
-    if isinstance(function, _TYPE_TYPE):
-        bound = True
-        function = function.__init__
-    elif isinstance(function, (types.FunctionType, types.MethodType)):
-        bound = is_bound_method(function)
-        function = getattr(function, '__wrapped__', function)
-    else:
-        function = function.__call__
-        bound = is_bound_method(function)
-    return inspect.getargspec(function), bound
-
-
 def get_callable_args(function, required_only=False):
     """Get names of callable arguments.
 
@@ -195,16 +230,17 @@ def get_callable_args(function, required_only=False):
     If required_only is True, optional arguments (with default values)
     are not included into output.
     """
-    argspec, bound = _get_arg_spec(function)
-    f_args = argspec.args
-    if required_only and argspec.defaults:
-        f_args = f_args[:-len(argspec.defaults)]
-    if bound:
-        f_args = f_args[1:]
-    return f_args
+    sig = get_signature(function)
+    function_args = list(six.iterkeys(sig.parameters))
+    for param_name, p in six.iteritems(sig.parameters):
+        if (p.kind in (Parameter.VAR_POSITIONAL, Parameter.VAR_KEYWORD) or
+                (required_only and p.default is not Parameter.empty)):
+            function_args.remove(param_name)
+    return function_args
 
 
 def accepts_kwargs(function):
-    """Returns True if function accepts kwargs."""
-    argspec, _bound = _get_arg_spec(function)
-    return bool(argspec.keywords)
+    """Returns ``True`` if function accepts kwargs otherwise ``False``."""
+    sig = get_signature(function)
+    return any(p.kind == Parameter.VAR_KEYWORD
+               for p in six.itervalues(sig.parameters))
