@@ -15,9 +15,9 @@
 
 import contextlib
 import socket
+from unittest import mock
 
-import mock
-from mock import patch
+import netaddr
 import netifaces
 from oslotest import base as test_base
 import six
@@ -95,7 +95,7 @@ class NetworkUtilsTest(test_base.BaseTestCase):
         self.assertEqual(result.netloc, 'user:pass@[::1]')
         self.assertEqual(result.path, '/v2.0/')
         self.assertEqual(result.hostname, '::1')
-        self.assertEqual(result.port, None)
+        self.assertIsNone(result.port)
 
         ipv6_url = 'https://[2001:db8:85a3::8a2e:370:7334]:1234/v2.0/xy?ab#12'
         result = netutils.urlsplit(ipv6_url)
@@ -164,30 +164,76 @@ class NetworkUtilsTest(test_base.BaseTestCase):
     def test_is_valid_ipv6(self):
         self.assertTrue(netutils.is_valid_ipv6('::1'))
 
+        self.assertTrue(netutils.is_valid_ipv6('fe80::1%eth0'))
+
+        self.assertFalse(netutils.is_valid_ip('fe%80::1%eth0'))
+
         self.assertFalse(netutils.is_valid_ipv6(
             '1fff::a88:85a3::172.31.128.1'))
 
         self.assertFalse(netutils.is_valid_ipv6(''))
 
+    def test_escape_ipv6(self):
+        self.assertEqual('[1234::1234]', netutils.escape_ipv6('1234::1234'))
+        self.assertEqual('127.0.0.1', netutils.escape_ipv6('127.0.0.1'))
+
     def test_is_valid_ip(self):
         self.assertTrue(netutils.is_valid_ip('127.0.0.1'))
 
         self.assertTrue(netutils.is_valid_ip('2001:db8::ff00:42:8329'))
 
+        self.assertTrue(netutils.is_valid_ip('fe80::1%eth0'))
+
         self.assertFalse(netutils.is_valid_ip('256.0.0.0'))
 
         self.assertFalse(netutils.is_valid_ip('::1.2.3.'))
 
         self.assertFalse(netutils.is_valid_ip(''))
 
+        self.assertFalse(netutils.is_valid_ip(None))
+
+    def test_is_valid_mac(self):
+        self.assertTrue(netutils.is_valid_mac("52:54:00:cf:2d:31"))
+        self.assertTrue(netutils.is_valid_mac(u"52:54:00:cf:2d:31"))
+        self.assertFalse(netutils.is_valid_mac("127.0.0.1"))
+        self.assertFalse(netutils.is_valid_mac("not:a:mac:address"))
+        self.assertFalse(netutils.is_valid_mac("52-54-00-cf-2d-31"))
+        self.assertFalse(netutils.is_valid_mac("aa bb cc dd ee ff"))
+        self.assertTrue(netutils.is_valid_mac("AA:BB:CC:DD:EE:FF"))
+        self.assertFalse(netutils.is_valid_mac("AA BB CC DD EE FF"))
+        self.assertFalse(netutils.is_valid_mac("AA-BB-CC-DD-EE-FF"))
+
+    def test_is_valid_cidr(self):
+        self.assertTrue(netutils.is_valid_cidr('10.0.0.0/24'))
+        self.assertTrue(netutils.is_valid_cidr('10.0.0.1/32'))
+        self.assertTrue(netutils.is_valid_cidr('0.0.0.0/0'))
+        self.assertTrue(netutils.is_valid_cidr('2600::/64'))
+        self.assertTrue(netutils.is_valid_cidr(
+                        '0000:0000:0000:0000:0000:0000:0000:0001/32'))
+
+        self.assertFalse(netutils.is_valid_cidr('10.0.0.1'))
+        self.assertFalse(netutils.is_valid_cidr('10.0.0.1/33'))
+        self.assertFalse(netutils.is_valid_cidr(10))
+
+    def test_is_valid_ipv6_cidr(self):
+        self.assertTrue(netutils.is_valid_ipv6_cidr("2600::/64"))
+        self.assertTrue(netutils.is_valid_ipv6_cidr(
+            "abcd:ef01:2345:6789:abcd:ef01:192.168.254.254/48"))
+        self.assertTrue(netutils.is_valid_ipv6_cidr(
+            "0000:0000:0000:0000:0000:0000:0000:0001/32"))
+        self.assertTrue(netutils.is_valid_ipv6_cidr(
+            "0000:0000:0000:0000:0000:0000:0000:0001"))
+        self.assertFalse(netutils.is_valid_ipv6_cidr("foo"))
+        self.assertFalse(netutils.is_valid_ipv6_cidr("127.0.0.1"))
+
     def test_valid_port(self):
-        valid_inputs = [1, '1', 2, '3', '5', 8, 13, 21,
+        valid_inputs = [0, '0', 1, '1', 2, '3', '5', 8, 13, 21,
                         '80', '3246', '65535']
         for input_str in valid_inputs:
             self.assertTrue(netutils.is_valid_port(input_str))
 
     def test_valid_port_fail(self):
-        invalid_inputs = ['-32768', '0', 0, '65536', 528491, '528491',
+        invalid_inputs = ['-32768', '65536', 528491, '528491',
                           '528.491', 'thirty-seven', None]
         for input_str in invalid_inputs:
             self.assertFalse(netutils.is_valid_port(input_str))
@@ -249,7 +295,7 @@ class NetworkUtilsTest(test_base.BaseTestCase):
     @mock.patch('netifaces.ifaddresses')
     def test_get_my_ipv4_address_with_default_route(
             self, ifaddr, gateways):
-        with patch.dict(netifaces.__dict__, {'AF_INET': '0'}):
+        with mock.patch.dict(netifaces.__dict__, {'AF_INET': '0'}):
             ifaddr.return_value = {'0': [{'addr': '172.18.204.1'}]}
             addr = netutils._get_my_ipv4_address()
         self.assertEqual('172.18.204.1', addr)
@@ -258,7 +304,7 @@ class NetworkUtilsTest(test_base.BaseTestCase):
     @mock.patch('netifaces.ifaddresses')
     def test_get_my_ipv4_address_without_default_route(
             self, ifaddr, gateways):
-        with patch.dict(netifaces.__dict__, {'AF_INET': '0'}):
+        with mock.patch.dict(netifaces.__dict__, {'AF_INET': '0'}):
             ifaddr.return_value = {}
             addr = netutils._get_my_ipv4_address()
         self.assertEqual('127.0.0.1', addr)
@@ -305,6 +351,44 @@ class IPv6byEUI64TestCase(test_base.BaseTestCase):
         self.assertRaises(TypeError, lambda:
                           netutils.get_ipv6_addr_by_EUI64(prefix, mac))
 
+    def test_generate_IPv6_with_empty_prefix(self):
+        mac = '00:16:3e:33:44:55'
+        prefix = ''
+        self.assertRaises(ValueError, lambda:
+                          netutils.get_ipv6_addr_by_EUI64(prefix, mac))
+
+
+class MACbyIPv6TestCase(test_base.BaseTestCase):
+    """Unit tests to extract MAC from IPv6."""
+
+    def test_reverse_generate_IPv6_by_EUI64(self):
+        self.assertEqual(
+            netaddr.EUI('00:16:3e:33:44:55'),
+            netutils.get_mac_addr_by_ipv6(
+                netaddr.IPAddress('2001:db8::216:3eff:fe33:4455')),
+        )
+
+    def test_random_qemu_mac(self):
+        self.assertEqual(
+            netaddr.EUI('52:54:00:42:02:19'),
+            netutils.get_mac_addr_by_ipv6(
+                netaddr.IPAddress('fe80::5054:ff:fe42:219')),
+        )
+
+    def test_local(self):
+        self.assertEqual(
+            netaddr.EUI('02:00:00:00:00:00'),
+            netutils.get_mac_addr_by_ipv6(
+                netaddr.IPAddress('fe80::ff:fe00:0')),
+        )
+
+    def test_universal(self):
+        self.assertEqual(
+            netaddr.EUI('00:00:00:00:00:00'),
+            netutils.get_mac_addr_by_ipv6(
+                netaddr.IPAddress('fe80::200:ff:fe00:0')),
+        )
+
 
 @contextlib.contextmanager
 def mock_file_content(content):
