@@ -24,7 +24,6 @@ import time
 
 from debtcollector import removals
 import iso8601
-from monotonic import monotonic as now  # noqa
 import pytz
 import six
 
@@ -37,6 +36,8 @@ PERFECT_TIME_FORMAT = _ISO8601_TIME_FORMAT_SUBSECOND
 
 _MAX_DATETIME_SEC = 59
 
+now = time.monotonic
+
 
 @removals.remove(
     message="use datetime.datetime.isoformat()",
@@ -55,7 +56,8 @@ def isotime(at=None, subsecond=False):
                      if not subsecond
                      else _ISO8601_TIME_FORMAT_SUBSECOND)
     tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
-    st += ('Z' if tz == 'UTC' else tz)
+    # Need to handle either iso8601 or python UTC format
+    st += ('Z' if tz in ('UTC', 'UTC+00:00') else tz)
     return st
 
 
@@ -256,7 +258,9 @@ def marshall_now(now=None):
              minute=now.minute, second=now.second,
              microsecond=now.microsecond)
     if now.tzinfo:
-        d['tzname'] = now.tzinfo.tzname(None)
+        # Need to handle either iso8601 or python UTC format
+        tzname = now.tzinfo.tzname(None)
+        d['tzname'] = 'UTC' if tzname == 'UTC+00:00' else tzname
     return d
 
 
@@ -283,6 +287,8 @@ def unmarshall_time(tyme):
                            microsecond=tyme['microsecond'])
     tzname = tyme.get('tzname')
     if tzname:
+        # Need to handle either iso8601 or python UTC format
+        tzname = 'UTC' if tzname == 'UTC+00:00' else tzname
         tzinfo = pytz.timezone(tzname)
         dt = tzinfo.localize(dt)
     return dt
@@ -401,7 +407,7 @@ class StopWatch(object):
     monotonically increasing time providing function (which typically varies
     depending on operating system and python version).
 
-    .. _monotonic: https://pypi.python.org/pypi/monotonic/
+    .. _monotonic: https://pypi.org/project/monotonic/
 
     .. versionadded:: 1.4
     """
@@ -416,7 +422,7 @@ class StopWatch(object):
         self._started_at = None
         self._stopped_at = None
         self._state = None
-        self._splits = []
+        self._splits = ()
 
     def start(self):
         """Starts the watch (if not already started).
@@ -428,13 +434,13 @@ class StopWatch(object):
         self._started_at = now()
         self._stopped_at = None
         self._state = self._STARTED
-        self._splits = []
+        self._splits = ()
         return self
 
     @property
     def splits(self):
         """Accessor to all/any splits that have been captured."""
-        return tuple(self._splits)
+        return self._splits
 
     def split(self):
         """Captures a split/elapsed since start time (and doesn't stop)."""
@@ -444,7 +450,7 @@ class StopWatch(object):
                 length = self._delta_seconds(self._splits[-1].elapsed, elapsed)
             else:
                 length = elapsed
-            self._splits.append(Split(elapsed, length))
+            self._splits = self._splits + (Split(elapsed, length),)
             return self._splits[-1]
         else:
             raise RuntimeError("Can not create a split time of a stopwatch"
