@@ -16,8 +16,6 @@
 """Performance tests for mask_password.
 """
 
-from __future__ import print_function
-
 import timeit
 
 from oslo_utils import strutils
@@ -37,7 +35,7 @@ print('payload has %d bytes' % len(input_str))
 for pattern in strutils._SANITIZE_PATTERNS_2['admin_pass']:
     print('\ntesting %s' % pattern.pattern)
     t = timeit.Timer(
-        "re.sub(pattern, r'\g<1>***\g<2>', payload)",
+        r"re.sub(pattern, r'\g<1>***\g<2>', payload)",
         """
 import re
 payload = '''%s'''
